@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -240,7 +240,8 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     ThrowReaderException(CorruptImageError,"FileFormatVersionMismatch");
   if (header.header_size < sz_XWDheader)
     ThrowReaderException(CorruptImageError,"CorruptImage");
-  switch (header.visual_class) {
+  switch (header.visual_class)
+  {
     case StaticGray:
     case GrayScale:
     case StaticColor:
@@ -250,8 +251,9 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     default:
       ThrowReaderException(CorruptImageError,"CorruptImage");
-    }
-  switch (header.pixmap_format) {
+  }
+  switch (header.pixmap_format)
+  {
     case XYBitmap:
     case XYPixmap:
     case ZPixmap:
@@ -481,17 +483,16 @@ static Image *ReadXWDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (x=0; x < (ssize_t) image->columns; x++)
             {
               pixel=XGetPixel(ximage,(int) x,(int) y);
-              index=(IndexPacket) ((pixel >> red_shift) & red_mask);
-              if (index < header.ncolors)
-                SetPixelRed(q,ScaleShortToQuantum(colors[(ssize_t) index].red));
-              index=(IndexPacket) ((pixel >> green_shift) & green_mask);
-              if (index < header.ncolors)
-                SetPixelGreen(q,ScaleShortToQuantum(colors[(ssize_t)
-                  index].green));
-              index=(IndexPacket) ((pixel >> blue_shift) & blue_mask);
-              if (index < header.ncolors)
-                SetPixelBlue(q,ScaleShortToQuantum(colors[(ssize_t)
-                  index].blue));
+              index=ConstrainColormapIndex(image,(pixel >> red_shift) &
+                red_mask);
+              SetPixelRed(q,ScaleShortToQuantum(colors[(ssize_t) index].red));
+              index=ConstrainColormapIndex(image,(pixel >> green_shift) &
+                green_mask);
+              SetPixelGreen(q,ScaleShortToQuantum(colors[(ssize_t)
+                index].green));
+              index=ConstrainColormapIndex(image,(pixel >> blue_shift) &
+                blue_mask);
+              SetPixelBlue(q,ScaleShortToQuantum(colors[(ssize_t) index].blue));
               q++;
             }
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
